(*
   Copyright 2012 by Mark Weyer
*)

let data_per_block = 32768
let extra_per_block = 5
let fullblocksize = data_per_block + extra_per_block

let deflate_block last inbuf inpos outbuf outpos len =
  outbuf.[outpos] <- if last then '\001' else '\000';
  outbuf.[outpos+1] <- char_of_int (len mod 256);
  outbuf.[outpos+2] <- char_of_int (len/256);
  outbuf.[outpos+3] <- char_of_int (255-len mod 256);
  outbuf.[outpos+4] <- char_of_int (255-len/256);
  String.blit inbuf inpos outbuf (outpos+5) len

let deflate last indata =
  let size = String.length indata  in
  let fullblocks = size / data_per_block  in
  let last_size = size mod data_per_block  in
  let blocks = fullblocks + if last_size>0 then 1 else 0  in
  let outdata = String.create (size + blocks * extra_per_block)  in
  for i=0 to fullblocks-1 do
    deflate_block (i=fullblocks-1 && last_size=0 && last)
      indata (data_per_block*i)
      outdata (fullblocksize*i)
      data_per_block;
  done;
  if last_size>0
  then deflate_block last
    indata (data_per_block*fullblocks)
    outdata (fullblocksize*fullblocks)
    last_size;
  outdata


class encoder = object
  val mutable s1 = 1
  val mutable s2 = 0

  method more_data last data =
    String.iter (fun c ->
        s1 <- (s1+int_of_char c) mod 65521;
        s2 <- (s2+s1) mod 65521)
      data;
    deflate last data

  method trailer =
    let s = "...."  in
    s.[0] <- char_of_int (s2/256);
    s.[1] <- char_of_int (s2 mod 256);
    s.[2] <- char_of_int (s1/256);
    s.[3] <- char_of_int (s1 mod 256);
    s
end

let encode _ = "x\001", new encoder

